#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/drivers/spi.h>
#include "lsm6dsv16x_reg.h"
#include <zephyr/devicetree.h>
#include "lsm6dsv16x_zephyr.h"

#include <zephyr/device.h>
#include <zephyr/drivers/spi.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/logging/log.h>
#include "lsm6dsv16x_reg.h"

LOG_MODULE_REGISTER(lsm6dsv16x, LOG_LEVEL_INF);

struct lsm6dsv16x_data {
    // Sensor data structures
};

struct lsm6dsv16x_config {
    struct spi_dt_spec spi;
};

static int lsm6dsv16x_who_am_i(const struct device *dev, uint8_t *id) {
    const struct lsm6dsv16x_config *cfg = dev->config;
    uint8_t tx_buf = LSM6DSV16X_WHO_AM_I;
    struct spi_buf tx = {
        .buf = &tx_buf,
        .len = 1,
    };
    struct spi_buf rx = {
        .buf = id,
        .len = 1,
    };
    struct spi_buf_set tx_set = {
        .buffers = &tx,
        .count = 1,
    };
    struct spi_buf_set rx_set = {
        .buffers = &rx,
        .count = 1,
    };

    int ret = spi_transceive_dt(&cfg->spi, &tx_set, &rx_set);
    if (ret < 0) {
        LOG_ERR("Failed to read WHO_AM_I register");
        return ret;
    }

    LOG_INF("WHO_AM_I: 0x%02x", *id);
    return 0;
}

static int lsm6dsv16x_init(const struct device *dev) {
    const struct lsm6dsv16x_config *cfg = dev->config;

    if (!device_is_ready(cfg->spi.bus)) {
        LOG_ERR("SPI bus is not ready");
        return -ENODEV;
    }

    uint8_t who_am_i;
    int ret = lsm6dsv16x_who_am_i(dev, &who_am_i);
    if (ret < 0) {
        return ret;
    }

    if (who_am_i != LSM6DSV16X_ID) {
        LOG_ERR("Unexpected WHO_AM_I value: 0x%02x", who_am_i);
        return -EINVAL;
    }

    return 0;
}

#define LSM6DSV16X_DEFINE(inst)                               \
    static const struct lsm6dsv16x_config lsm6dsv16x_config_##inst = { \
        .spi = SPI_DT_SPEC_INST_GET(inst, SPI_WORD_SET(8) | SPI_TRANSFER_MSB, 0) \
    };                                                              \
    DEVICE_DT_INST_DEFINE(inst,                                     \
                          lsm6dsv16x_init,                   \
                          NULL,                                     \
                          NULL,                                     \
                          &lsm6dsv16x_config_##inst,         \
                          POST_KERNEL,                              \
                          CONFIG_SENSOR_INIT_PRIORITY,              \
                          NULL);

DT_INST_FOREACH_STATUS_OKAY(LSM6DSV16X_DEFINE)
